/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerDateModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xnap.commons.gui.DefaultDialog;
import org.xnap.commons.gui.Dialogs;
import org.xnap.commons.gui.action.AbstractXNapAction;
import org.xnap.commons.i18n.I18n;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

public abstract class AbstractFaxDialog extends DefaultDialog implements LocaleChangeListener {

	private final static Log logger = LogFactory.getLog(QueuePanel.class);
	public static final int DEFAULT_COLUMNS = 20;
	
	private JobPanel jobPanel;
	private SpinnerDateModel dateModel;
	private HylaFAXJob job;
	private JLabel dateLabel;
	private ParameterDialogAction parameterAction;
	private JRadioButton dateNowRadionButton;
	private JRadioButton dateLaterRadionButton;
	protected DefaultFormBuilder builder;
	private JTextField numberTextField;
	private JLabel numberLabel;
	
	public AbstractFaxDialog() {
		super(BUTTON_OKAY | BUTTON_CANCEL);
	
		FormLayout layout = new FormLayout("right:max(40dlu;pref), 3dlu, pref, 3dlu, min, min:grow, 3dlu, min", "");
		builder = new DefaultFormBuilder(layout);
		builder.setDefaultDialogBorder();
		setMainComponent(builder.getPanel());
		
		parameterAction = new ParameterDialogAction();
		getButtonPanel().add(parameterAction.createJButton(), 0);
	}

	protected void addNumberTextField() {
		numberTextField = new JTextField();
		numberLabel = builder.append("", numberTextField, 6);
		builder.nextLine();
	}			

	protected void addDateControls() {
		dateNowRadionButton = new JRadioButton();
		dateNowRadionButton.setSelected(true);
		dateLabel = builder.append("", dateNowRadionButton);
		builder.nextLine();
		
		dateLaterRadionButton = new JRadioButton();
		dateModel = new SpinnerDateModel();
		final JSpinner dateSpinner = new JSpinner(dateModel);
		dateSpinner.setEnabled(false);
		dateLaterRadionButton.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				dateSpinner.setEnabled(dateLaterRadionButton.isSelected());
			}			
		});
		builder.append("", dateLaterRadionButton, dateSpinner);
		builder.nextLine();
		
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(dateNowRadionButton);
		buttonGroup.add(dateLaterRadionButton);		
	}
	
	@Override
	public boolean apply() {
		if (numberTextField != null) {
			if (numberTextField.getText().trim().length() == 0) {
				Dialogs.showError(this, I18n.tr("Please enter a number"), 
						I18n.tr("JHylaFAX Error"));
				numberTextField.requestFocus();
				return false;
			}
			getJob().setNumber(numberTextField.getText());
		}
		
		if (dateNowRadionButton != null) {
			job.setSendTime((dateNowRadionButton.isSelected()) 
					? null 
					: dateModel.getDate());
		}
		return true;
	}
	
	public HylaFAXJob getJob() {
		return job;
	}
	
	public void updateLabels() {
		if (numberTextField != null) {
			numberLabel.setText(I18n.tr("Number"));
		}
		if (dateNowRadionButton != null) {
			dateLabel.setText(I18n.tr("Date"));
			dateNowRadionButton.setText(I18n.tr("Now"));
			dateLaterRadionButton.setText(I18n.tr("Later"));
		}		
		parameterAction.updateLabels();
	}
	
	public void revert() {
		if (numberTextField != null) {
			numberTextField.setText(getJob().getNumber());
		}
		if (dateNowRadionButton != null) {
			if (getJob().getSendTime() == null) {
				dateNowRadionButton.setSelected(true);
			}
			else {
				dateLaterRadionButton.setSelected(true);
				dateModel.setValue(job.getSendTime());
			}
		}
	}

	public void setJob(HylaFAXJob job) {
		this.job = job;
	}
	
	private class ParameterDialogAction extends AbstractXNapAction implements LocaleChangeListener{
		
		private JobDialog dialog;
		
		public ParameterDialogAction() {
			//putValue(ICON_FILENAME, "configure.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			if (job == null) {
				return;
			}
			
			if (dialog == null) {
				dialog = new JobDialog(job);
				dialog.setLocationRelativeTo(AbstractFaxDialog.this);
				dialog.setModal(true);
			}
			
			if (!dialog.isVisible()) {
				dialog.revert();
				dialog.getJobPanel().setSenderEditable(getJob().getID() == -1);
				dialog.setVisible(true);
			}
		}
		
		public void updateLabels() {
			parameterAction.putValue(Action.NAME, I18n.tr("Parameter..."));
			parameterAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Opens a dialog for entering job parameter"));

			if (dialog != null) {
				dialog.updateLabels();
			}
		}
		
	}

}
