/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import gnu.hylafax.HylaFAXClient;
import java.awt.event.ActionEvent;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xnap.commons.gui.ErrorDialog;
import org.xnap.commons.gui.action.AbstractXNapAction;
import org.xnap.commons.i18n.I18n;
import org.xnap.commons.io.Job;
import org.xnap.commons.io.ProgressMonitor;

/**
 * A dialog for sending faxes.
 * 
 * @author Steffen Pingel
 */
public class SendDialog extends AbstractFaxDialog {

	private final static Log logger = LogFactory.getLog(QueuePanel.class);

	private JTextField documentTextField;
	private JLabel documentLabel;
	private FileDialogAction fileDialogAction;
	
	public SendDialog() {
		addNumberTextField();
		
		documentTextField = new JTextField();
		documentLabel = builder.append("", documentTextField, 4);
		fileDialogAction = new FileDialogAction();		
		builder.append(fileDialogAction.createIconButton());
		builder.nextLine();

		addDateControls();
		
		HylaFAXJob job = new HylaFAXJob(); 
		HylaFAXClientHelper.initializeFromSettings(job);
		setJob(job);
		
		updateLabels();
		pack();
	}
	
	@Override
	public boolean apply() {
		if (!super.apply()) {
			return false;
		}
		final File file = new File(documentTextField.getText()); 
			
		Job<?> ioJob = new Job() {
			@Override
			public Object run(ProgressMonitor monitor) throws Exception
			{
				monitor.setTotalSteps(7);
				
				HylaFAXClient client = JHylaFAX.getInstance().getConnection(monitor);
				monitor.work(1);
				
				client.mode(HylaFAXClient.MODE_STREAM);
				client.type(HylaFAXClient.TYPE_IMAGE);
				
				String serverFilename;
				InputStream in = new BufferedInputStream(new FileInputStream(file));
				try {
					serverFilename = client.putTemporary(in);
				}
				finally {
					in.close();
				}
				monitor.work(2);

//				if (request.getCover() != null) {
//				FaxCover faxCover = new FaxCover(request.getCoverFilename(), request.getJob().getFilename(), request.getCover());
//
//			    // Cover generieren
//			    faxCover.generate();
//
//			    // Cover senden
//			    String cover = getConnection().sendbuffer(faxCover.getCharArray());
//			    if(debug) System.out.println("SuSEFax.callSendfax(COVER): " + cover);
//			    getConnection().command("JPARM COVER " + cover);
//			}
				monitor.work(2);

				gnu.hylafax.Job sendJob = client.createJob();
				HylaFAXClientHelper.applyParameter(sendJob, getJob());
				sendJob.addDocument(serverFilename);
				monitor.work(1);

				
				client.submit(sendJob);
				monitor.work(1);
				
				return null;
			}
		};
		
		try {
			JHylaFAX.getInstance().runJob(ioJob);
		} 
		catch (UserAbortException e) {
			return false;
		}
		catch (Exception e) {
			logger.debug("Error sending fax", e);
			ErrorDialog.showError(this, I18n.tr("Could not send fax"), 
					I18n.tr("JHylaFAX Error"),
					e);
			return false;
		}
		return true;
	}
	
	public void updateLabels() {
		super.updateLabels();

		setTitle(I18n.tr("Send Fax"));
		documentLabel.setText(I18n.tr("Document"));
	}

	private class FileDialogAction extends AbstractXNapAction {
		
		public FileDialogAction() {
			putValue(ICON_FILENAME, "folder_open.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			JFileChooser dialog = new JFileChooser();
			dialog.setSelectedFile(new File(documentTextField.getText()));
			if (dialog.showOpenDialog(SendDialog.this) == JFileChooser.APPROVE_OPTION) {
				documentTextField.setText(dialog.getSelectedFile().getAbsolutePath());
			}
		}
		
	}

}
